#Copyright (c) 2008, MANDIANT Corporation
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#    * Neither the name of the MANDIANT Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
#INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
#DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
#SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
#WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
#USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
@author:       Peter Silberman
@license:      BSD  
@contact:      peter.silberman@mandiant.com
@organization: http://www.mandiant.com
'''

import os
import sys
import re
import urllib
import getopt
from parsesnort import *
from output import *

USAGE = "\nWritten by Peter Silberman (peter.silberman@mandiant.com)\n" \
	"\nUSAGE: mindsniffer.py "                                                            	\
        "\n    <-r|--rules RULE FILE>  snort rule file to parse"                 		\
        "\n    <-x|--xpath>            generate xpath filter"       				\
        "\n    <-p|--py>               generate py files for use in AuditViewer"              	\
        "\n    [-n|--nand]             use \'or\' in xpath filter generation"              	\
        "\n    [-m|--memory]           use inconjunction with -x specify a memory image to generate xpath for" \
        "\n    [-o|--output]           specify output directory"              	




    
if len(sys.argv) < 3:
    print USAGE
    sys.exit(1)


try:
    opts, args = getopt.getopt(sys.argv[1:], "r:o:m:xpn", \
        ["rules=", "xpath", "py", "output=","nand","memory="])
except getopt.GetoptError, error:
    print str(error)
    print USAGE
    sys.exit(1)


generate_py = generate_xpath = nand = False
output_path = url = rule_file = memory_file =  None


for opt, arg in opts:
    if opt in ("-r", "--rules"):      	rule_file          	= arg
    if opt in ("-x", "--xpath"):      	generate_xpath          = True
    if opt in ("-p", "--py"):     	generate_py         	= True
    if opt in ("-o", "--output"):       output_path   		= arg
    if opt in ("-n", "--nand"):     	nand         		= True
    if opt in ("-m", "--memory"):     	memory_file         	= arg


if ( (not rule_file) or (not generate_xpath and not generate_py)):
    print USAGE
    sys.exit(1)

if not output_path:
    output_path = os.getcwd() + "\\"

if not output_path.endswith("\\"):
    output_path += "\\"

fd = open(rule_file, "r")
ignored = 0
created = 0
ret     = 0
total   = 0
for line in fd.readlines():
    line = line.strip()

    if not len(line) or line[0] == '#':
        pass
    elif len(line) > 1:
        ret = 0
        sig = ParseRule(line)
        if generate_xpath:
            ret = WriteXPathSignature(sig, output_path, nand, memory_file)
        elif generate_py:
            ret = WritePythonSignature(sig, output_path)
        if ret:
            created+=1
        else:
            ignored+=1
        total+=1

print "Total signatures read: %d" % total
print "Created signatures: %d" % created
print "Ignored signatures: %d" % ignored
print "Success rate: %f" % float(float(created)/float(total))