#Copyright (c) 2008, MANDIANT Corporation
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#    * Neither the name of the MANDIANT Corporation nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
#INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
#DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
#SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
#SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
#WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
#USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
@author:       Peter Silberman
@license:      BSD  
@contact:      peter.silberman@mandiant.com
@organization: http://www.mandiant.com
'''

import re

dbg = 0

def GetSignatureName(line):
    '''
    Get the name of the signature we use the msg: as our name
    '''
    p = re.compile(".*msg:\"([^\"]*)")
    m = p.match(line)
    if m:
        sig_name = m.group(1)
        return sig_name
    p = re.compile(".*msg: \"([^\"]*)")
    m = p.match(line)
    if m:
        sig_name = m.group(1)
 
        return sig_name
    return None


def SantizeContents(l):
    '''
    This function normalizes any alphanumeric (and some printable) characters between | | if we hit a non alphanumeric character then we return None
    '''
    result = ""
    fail = 0
    for element in l:
        if len(element):
            i = 0
            q = ""
            if element[0] == "|" and element[len(element)-1] == "|":

                element = element.replace("00", "").replace(" ", "").replace("|", "")
                while i < len(element):
                    #
                    # 2 bytes in c
                    #
                    c = element[i:i+2]
                    #
                    # if c is within 0x20 and 0x7E its 
                    # a valid ascii character
                    #

                    if int(c, 16) == 0x0A or int(c, 16) == 0x0D:
                        i+=2
                        continue
                        
                    if int(c, 16) < 0x20 or int(c, 16) > 0x7E:
                        fail = 1
                        break

            
                    
                    q+= chr( int(c, 16) )
                    i+=2
                if fail:
                    return None
                if q != "":
                    result += q                    
            else:
                result += element
    return result
                    
def SnortSplit(l):
    '''
    split the signature up based on | | each element in the list is either before after or contains | |
    '''
    i = 0
    result = []
    leftover = ""
    while i < len(l):
        if l[i] == "|":
            if i != 0:
                leftover = l[:i]
            end = l.find("|", i+1)
            s = l[i:end+1]
            result.append(leftover)
            result.append(s)
            i = 0
            l = l[end+1:]
            continue    
        i+=1
    if len(l):
        result.append(l)
    return result

def BreakSignatureOnNewline(l):
    '''
    Our XML santizer when doing strings seperates the string on \r or \n doesn't matter. So we have to break the signature into different pieces if it contains
    these characters
    '''
    i = 0
    split_signatures = []
    leftover = ""
    while i < len(l):
        if l[i] == "|":
            if i != 0:
                leftover = l[:i]
            end = l.find("|", i+1)
            s = l[i:end+1]
            if s.lower() == "|0a 0d|" or s.lower() == "|0d 0a|":
                split_signatures.append(leftover)

                i = 0
                l = l[end+1:]
                continue    
        i+=1
    if len(l):
        split_signatures.append(l)
    return split_signatures
    

def GetSignatureContents(line):
    '''
    Get the content we parse it either content contains | | or it doesn't
    '''
    p = re.compile("content:\"([^\"]*)")
    m = p.findall(line)
    ContentLists = []
    
    
    if m:
        for l in m:
            if l.find("|") != -1:
                b = BreakSignatureOnNewline(l)
                
                for li in b:
                    sp = SnortSplit(li)
                    contents = SantizeContents(sp)
                    if contents:

                        ContentLists.append( contents )
            else:
                if l:
                    ContentLists.append( l )

                
        if len(ContentLists):
            return ContentLists
    return None

def GetSignaturePCRE(line):
    '''
    grab the PCRE command
    '''
    p = re.compile("pcre:\"([^\;]*)")
    m = p.findall(line)
    PCRELists = []
    if m:
        for l in m:
            l = l[:-1]
            #
            # sanity check to see if pcre contains /regex/ or just regex
            #
            if l[0] == "/":
                l = l[l.find("/")+1:l.rfind("/")]
            PCRELists.append(l)
        return PCRELists
    
    return None
    

def ParseRule(line):
    Signature = {}
    Signature["SignatureName"] = GetSignatureName(line)

    Signature["SignatureContents"] = GetSignatureContents(line)
    Signature["SignaturePCRE"] = GetSignaturePCRE(line)    

    
    return Signature
